@@ -7,9 +7,9 @@ from plotdevice import DeviceError
 from . import _cg_context
 from .atoms import PenMixin, TransformMixin, ColorMixin, EffectsMixin, Grob
 from .colors import Color, Gradient, Pattern
-from .transform import CENTER, Transform, Region, Size, Point, DEGREES
+from .geometry import CENTER, DEGREES, Transform, Region, Point
 from ..util import trim_zeroes, _copy_attr, _copy_attrs, _flatten, numlike
-from ..lib import pathmatics, geometry
+from ..lib import pathmatics
 
 _ctx = None
 __all__ = ("Bezier", "Curve", "BezierPath", "PathElement",
@@ -42,12 +42,13 @@ FORTYFIVE = "fortyfive"
 
 class Bezier(EffectsMixin, TransformMixin, ColorMixin, PenMixin, Grob):
     """A Bezier provides a wrapper around NSBezierPath."""
-    stateAttrs = ('_nsBezierPath',)
+    stateAttrs = ('_nsBezierPath', '_fulcrum')
+    opts = ('close', 'smooth')
 
-    def __init__(self, path=None, immediate=False, **kwargs):
+    def __init__(self, path=None, **kwargs):
         super(Bezier, self).__init__(**kwargs)
         self._segment_cache = {} # used by pathmatics
-        self._fulcrum = None # centerpoint (set only for primitives)
+        self._fulcrum = None # centerpoint (set only for center-based primitives)
 
         # path arg might contain a list of point tuples, a bezier to copy, or a raw
         # nsbezier reference to use as the backing store. otherwise start with a
@@ -62,22 +63,15 @@ class Bezier(EffectsMixin, TransformMixin, ColorMixin, PenMixin, Grob):
                 p = pathmatics.findpath(path, 1.0 if kwargs.get('smooth') else 0.0)
                 self._nsBezierPath = p._nsBezierPath
         elif isinstance(path, Bezier):
-            self.inherit(path)
+            _copy_attrs(path, self, Bezier.stateAttrs)
         elif isinstance(path, NSBezierPath):
-            self._nsBezierPath = path
+            self._nsBezierPath = path.copy()
         else:
             badpath = "Don't know what to do with %s." % path
             raise DeviceError(badpath)
 
         # decide what needs to be done at the end of the `with` context
-        self._autoclose = kwargs.get('close', False)
-        self._autodraw = kwargs.get('draw', False)
-
-        # finish the path (and potentially draw it) immediately if flagged to do so.
-        # in practice, `immediate` is only passed when invoked by the `bezier()` command
-        # with a preexisting point-set or bezier `path` argument.
-        if immediate and kwargs.get('draw',True):
-            self._autofinish()
+        self._needs_closure = kwargs.get('close', False)
 
     def __enter__(self):
         self._rollback = {attr:getattr(_ctx,attr) for attr in ['_path','_transform','_transformmode']}
@@ -86,21 +80,14 @@ class Bezier(EffectsMixin, TransformMixin, ColorMixin, PenMixin, Grob):
         return self
 
     def __exit__(self, type, value, tb):
-        self._autofinish()
+        self._autoclose()
         for attr, val in self._rollback.items():
             setattr(_ctx, attr, val)
 
-    def _autofinish(self):
-        if self._autoclose:
-            self.closepath()
-        if self._autodraw:
-            self.draw()
-        self._autoclose = self._autodraw = False
-
-    @property
-    def path(self):
-        warnings.warn("The 'path' attribute is deprecated. Please use _nsBezierPath instead.", DeprecationWarning, stacklevel=2)
-        return self._nsBezierPath
+    def copy(self):
+        clone = Bezier()
+        clone.inherit(self)
+        return clone
 
     ### Path methods ###
 
@@ -137,8 +124,8 @@ class Bezier(EffectsMixin, TransformMixin, ColorMixin, PenMixin, Grob):
 
             # ...and transform it to match the endpoints
             src = self._nsBezierPath.currentPoint()
-            theta = geometry.angle(src.x, src.y, x1, y1)
-            dw = geometry.distance(src.x, src.y, x1, y1)
+            theta = pathmatics.angle(src.x, src.y, x1, y1)
+            dw = pathmatics.distance(src.x, src.y, x1, y1)
             dh = dw*(-1.0 if ccw else 1.0)
             t = Transform()
             t.translate(src.x,src.y)
@@ -150,24 +137,10 @@ class Bezier(EffectsMixin, TransformMixin, ColorMixin, PenMixin, Grob):
     def closepath(self):
         self._nsBezierPath.closePath()
 
-    @property
-    def bounds(self):
-        try:
-            return Region(*self._nsBezierPath.bounds())
-        except:
-            # Path is empty -- no bounds
-            return Region()
-
-    @property
-    def center(self):
-        if self._fulcrum:
-            return Point(self._fulcrum)
-        else:
-            (x, y), (w, h) = self.bounds
-            return Point(x+w/2, y+h/2)
-
-    def contains(self, x, y):
-        return self._nsBezierPath.containsPoint_((x,y))
+    def _autoclose(self):
+        if self._needs_closure:
+            self.closepath()
+            self._needs_closure = False
 
     ### Basic shapes (origin + size) ###
 
@@ -224,7 +197,7 @@ class Bezier(EffectsMixin, TransformMixin, ColorMixin, PenMixin, Grob):
     def poly(self, x, y, radius, sides=4, points=None):
         # if `points` is defined, draw a regularized star, otherwise draw
         # a regular polygon with the given number of `sides`.
-        if points>4:
+        if points and points>4:
             inner = radius * cos(pi*2/points)/cos(pi/points)
             return self.star(x, y, points, inner, radius)
         elif points is not None:
@@ -243,7 +216,7 @@ class Bezier(EffectsMixin, TransformMixin, ColorMixin, PenMixin, Grob):
 
         # rotate the origin slightly so the polygon sits on an edge
         theta = pi/2 + (0 if sides%2 else 1*pi/sides)
-        angles = [2*pi * i/sides - theta for i in xrange(sides)]
+        angles = [2*pi * i/sides - theta for i in range(sides)]
 
         # walk around the circle adding points with proper scale/origin
         points = [ [radius*cos(theta)+x, radius*sin(theta)+y] for theta in angles]
@@ -286,6 +259,7 @@ class Bezier(EffectsMixin, TransformMixin, ColorMixin, PenMixin, Grob):
           pt = (x+radius*sin(angle), y+radius*cos(angle))
           self._nsBezierPath.lineToPoint_(pt)
         self.closepath()
+        self._fulcrum = Point(x,y)
 
     def arrow(self, x, y, width=100, type=NORMAL):
         if type not in (NORMAL, FORTYFIVE):
@@ -324,7 +298,7 @@ class Bezier(EffectsMixin, TransformMixin, ColorMixin, PenMixin, Grob):
     def __getitem__(self, index):
         if isinstance(index, slice):
             # slice-based access
-            pts = [self._nsBezierPath.elementAtIndex_associatedPoints_(i) for i in xrange(*index.indices(len(self)))]
+            pts = [self._nsBezierPath.elementAtIndex_associatedPoints_(i) for i in range(*index.indices(len(self)))]
             return [Curve(cmd, el) for cmd,el in pts]
         else:
             # index-based access
@@ -371,39 +345,45 @@ class Bezier(EffectsMixin, TransformMixin, ColorMixin, PenMixin, Grob):
 
     ### Drawing methods ###
 
+    @property
+    def bounds(self):
+        try:
+            return Region(self._nsBezierPath.bounds())
+        except:
+            # Path is empty -- no bounds
+            return Region()
+
+    @property
+    def center(self):
+        if self._fulcrum:
+            return Point(self._fulcrum)
+        else:
+            (x, y), (w, h) = self.bounds
+            return Point(x+w/2, y+h/2)
+
+    def contains(self, x, y):
+        return self._nsBezierPath.containsPoint_((x,y))
+
     @property
     def _screen_transform(self):
         """Returns the Transform object that will be used to draw the path."""
 
+        nudge = Transform()
         if (self.transformmode == CENTER):
             # if center-based, sandwich transform with a scoot out/in to the origin
-            dx, dy = self.center
-            nudge = Transform()
-            nudge.translate(-dx, -dy)
-            xf = self.transform.copy()
-            xf.prepend(nudge)
-            xf.append(nudge.inverse)
-            return xf
-        else:
-            return self.transform
+            dx, dy = self._to_px(self.center)
+            nudge.translate(dx, dy)
+
+        xf = Transform()
+        xf.prepend(nudge)
+        xf.prepend(self.transform)
+        xf.prepend(nudge.inverse)
+        return xf
 
     @property
     def cgPath(self):
-        # this really ought to live in pathmatics...
-        ns = self._nsBezierPath
-        cg = CGPathCreateMutable()
-        for cmd, points in (ns.elementAtIndex_associatedPoints_(i) for i in xrange(ns.elementCount())):
-            if cmd==NSMoveToBezierPathElement:
-                CGPathMoveToPoint(cg, None, points[0].x, points[0].y)
-            elif cmd==NSLineToBezierPathElement:
-                CGPathAddLineToPoint(cg, None, points[0].x, points[0].y)
-            elif cmd==NSCurveToBezierPathElement:
-                CGPathAddCurveToPoint(cg, None, points[0].x, points[0].y,
-                                                points[1].x, points[1].y,
-                                                points[2].x, points[2].y)
-            elif cmd==NSClosePathBezierPathElement:
-                CGPathCloseSubpath(cg)
-        return CGPathCreateCopy(cg)
+        # transform the path's points from canvas- to postscript-units and return a CGPathRef
+        return pathmatics.convert_path(self._to_px(self._nsBezierPath))
 
     def _draw(self):
         with _cg_context() as port:
@@ -474,9 +454,24 @@ class Bezier(EffectsMixin, TransformMixin, ColorMixin, PenMixin, Grob):
                 t.scale(min(width /pw, height / ph))
         t.translate(-px, -py)
         self._nsBezierPath = t.apply(self)._nsBezierPath
-        self._fulcrum = t.apply(self._fulcrum)
+        self._fulcrum = t.apply(self._fulcrum) if self._fulcrum else None
         self._segment_cache = {}
 
+    def _get_x(self):
+        return getattr(self._fulcrum or self.bounds.origin.x, 'x')
+    def _set_x(self, new_x):
+        if self._fulcrum:
+            new_x -= self._fulcrum.x - self.bounds.origin.x
+        self.fit(x=new_x)
+    x = property(_get_x, _set_x)
+
+    def _get_y(self):
+        return getattr(self._fulcrum or self.bounds.origin.x, 'y')
+    def _set_y(self, new_y):
+        if self._fulcrum:
+            new_y -= self._fulcrum.y - self.bounds.origin.y
+        self.fit(y=new_y)
+    y = property(_get_y, _set_y)
 
     ### Mathematics ###
 
@@ -503,7 +498,7 @@ class Bezier(EffectsMixin, TransformMixin, ColorMixin, PenMixin, Grob):
 
         count = int(amount) # make sure we don't choke on a float
         delta = 1.0/max(1, count-1) # div by count-1 so the last point is at t=1.0
-        for i in xrange(count):
+        for i in range(count):
             yield pathmatics.point(self, delta*i)
 
     def addpoint(self, t):
